import 'dart:async';
import 'dart:convert';

import 'package:http/http.dart' as http;
import 'package:mwb/bloc_provider.dart';
import 'package:mwb_sample_reactiveui/post.dart';
import 'package:rxdart/rxdart.dart';

class PostsPageBloc extends BaseBloc {
  /// Stream controller wrapper for all posts.
  ///
  /// A [Subject] is basically a simplified [StreamController] with a combination
  /// of a [Stream] and a [Sink] that are linked together.
  /// In this case a [BehaviorSubject] stores the latest value received over it's sink
  /// which is broadcasted to all listeners on subscription.
  ///
  /// If you only want to pass values along without caching them
  /// you can use a [PublishSubject].
  final BehaviorSubject<List<Post>> _postsSubject = BehaviorSubject.seeded([]);

  Stream<List<Post>> get postsStream => _postsSubject.stream;

  @override
  void init() {}

  @override
  Future dispose() async {
    await _postsSubject.drain();
    _postsSubject.close();
  }

  Future<List<Post>?> _getSamplePosts() async {
    final res =
        await http.get(Uri.parse("https://jsonplaceholder.typicode.com/posts"));
    final json = jsonDecode(res.body);
    if (json is List<dynamic>) {
      final list = json.map((el) => Post.fromJSON(el)).toList();
      return list;
    }
    return null;
  }

  Future refreshList() async {
    final posts = await _getSamplePosts();
    _postsSubject.sink.add(posts ?? []);
  }
}
